CREATE TABLE ARTIFACTS (AID INTEGER GENERATED BY DEFAULT AS IDENTITY,
                        PATH VARCHAR (500) NOT NULL,
                        MEDIA_TYPE VARCHAR (500),
                        DIRECTORY INTEGER NOT NULL,
                        STATE INTEGER,
                        AUTHOR VARCHAR (500),
                        CREATED_TIME TIMESTAMP,
                        LAST_UPDATER VARCHAR (500),
                        LAST_UPDATED_TIME TIMESTAMP,
                        DESCRIPTION VARCHAR (500),
                        CONTENT BINARY,
                        PRIMARY KEY (AID),
                        UNIQUE(PATH));

CREATE TABLE PROPERTIES (P_ID INTEGER GENERATED BY DEFAULT AS IDENTITY,
                            AID INTEGER NOT NULL,
                            PKEY VARCHAR (100) NOT NULL,
                            PVALUE VARCHAR (500) NOT NULL,
                            PRIMARY KEY (P_ID),
                            UNIQUE (AID, PKEY),
                            FOREIGN KEY (AID) REFERENCES ARTIFACTS (AID));

CREATE TABLE CHILDREN (CID INTEGER GENERATED BY DEFAULT AS IDENTITY,
                        PARENT_ID INTEGER NOT NULL,
                        CHILD_ID INTEGER NOT NULL,
                        PRIMARY KEY (CID),
                        UNIQUE (PARENT_ID, CHILD_ID),
                        FOREIGN KEY (PARENT_ID) REFERENCES ARTIFACTS (AID),
                        FOREIGN KEY (CHILD_ID) REFERENCES ARTIFACTS (AID));

CREATE TABLE TAGS (TN_ID INTEGER GENERATED BY DEFAULT AS IDENTITY,
                    TAG_NAME VARCHAR (500) NOT NULL,
                    AID INTEGER NOT NULL,
                    USER_ID VARCHAR (20) NOT NULL,
                    TAGGED_TIME TIMESTAMP NOT NULL,
                    PRIMARY KEY (TN_ID),
                    UNIQUE (TAG_NAME, AID, USER_ID),
                    FOREIGN KEY (AID) REFERENCES ARTIFACTS (AID));

CREATE TABLE COMMENTS (CM_ID INTEGER GENERATED BY DEFAULT AS IDENTITY,
                        AID INTEGER NOT NULL,
                        USER_ID VARCHAR (20) NOT NULL,
                        COMMENT_TEXT VARCHAR (500) NOT NULL,
                        COMMENTED_TIME TIMESTAMP NOT NULL,
                        PRIMARY KEY (CM_ID),
                        FOREIGN KEY (AID) REFERENCES ARTIFACTS (AID));

CREATE TABLE LOGS (LOG_ID INTEGER GENERATED BY DEFAULT AS IDENTITY,
                    RID INTEGER,
                    USER_ID VARCHAR (20) NOT NULL,
                    LOGGED_TIME TIMESTAMP NOT NULL,
                    ACTION INTEGER NOT NULL,
                    ACTION_DATA VARCHAR (500),
                    PRIMARY KEY (LOG_ID));

CREATE TABLE RATINGS (R_ID INTEGER GENERATED BY DEFAULT AS IDENTITY,
                        AID INTEGER NOT NULL,
                        USER_ID VARCHAR (20) NOT NULL,
                        RATING INTEGER NOT NULL,
                        RATED_TIME TIMESTAMP NOT NULL,
                        PRIMARY KEY (R_ID),
                        FOREIGN KEY (AID) REFERENCES ARTIFACTS (AID));

CREATE TABLE VERSIONS (AID INTEGER NOT NULL,
                           VN INTEGER NOT NULL,
                           CONTENT BINARY,
                           AUTHOR VARCHAR (500),
                           UPDATED_TIME TIMESTAMP,
                           UNIQUE (AID,VN),
                           FOREIGN KEY (AID) REFERENCES ARTIFACTS (AID));

CREATE TABLE DEPENDENCY (AID INTEGER NOT NULL,
                          VN INTEGER NOT NULL,
                          DAID INTEGER NOT NULL,
                          DVN INTEGER NOT NULL,
                          UNIQUE (AID,VN,DAID,DVN),
                          FOREIGN KEY (DAID) REFERENCES ARTIFACTS (AID),
                          FOREIGN KEY (AID) REFERENCES ARTIFACTS (AID));

CREATE TABLE UM_USERS (
			ID INTEGER GENERATED BY DEFAULT AS IDENTITY, 
			USER_NAME VARCHAR(255) NOT NULL, 
			PASSWORD VARCHAR(255) NOT NULL, 
			PRIMARY KEY (ID), 
			UNIQUE(USER_NAME));

CREATE TABLE UM_USER_ATTRIBUTES (
			ID INTEGER GENERATED BY DEFAULT AS IDENTITY, 
			ATTR_NAME VARCHAR(255) NOT NULL, 
			ATTR_VALUE VARCHAR(255), 
			USER_ID INTEGER, 
			FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE, 
			PRIMARY KEY (ID));

CREATE TABLE UM_ROLES (
			ID INTEGER GENERATED BY DEFAULT AS IDENTITY, 
			ROLE_NAME VARCHAR(255) NOT NULL, 
			PRIMARY KEY (ID), 
			UNIQUE(ROLE_NAME));

CREATE TABLE UM_ROLE_ATTRIBUTES (
			ID INTEGER GENERATED BY DEFAULT AS IDENTITY, 
			ATTR_NAME VARCHAR(255) NOT NULL, 
			ATTR_VALUE VARCHAR(255), 
			ROLE_ID INTEGER, 
			FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE, 
			PRIMARY KEY (ID));

CREATE TABLE UM_PERMISSIONS (
			ID INTEGER GENERATED BY DEFAULT AS IDENTITY, 
			RESOURCE_ID VARCHAR(255) NOT NULL, 
			ACTION VARCHAR(255) NOT NULL, 
			PRIMARY KEY (ID));

CREATE TABLE UM_ROLE_PERMISSIONS (
			ID INTEGER GENERATED BY DEFAULT AS IDENTITY, 
			PERMISSION_ID INTEGER NOT NULL, 
			ROLE_ID INTEGER NOT NULL, 
			IS_ALLOWED SMALLINT NOT NULL, 
			UNIQUE (PERMISSION_ID, ROLE_ID), 
			FOREIGN KEY (PERMISSION_ID) REFERENCES UM_PERMISSIONS(ID) ON DELETE  CASCADE, 
			FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE, 
			PRIMARY KEY (ID));

CREATE TABLE UM_USER_PERMISSIONS (
			ID INTEGER GENERATED BY DEFAULT AS IDENTITY, 
			PERMISSION_ID INTEGER NOT NULL, 
			USER_ID INTEGER NOT NULL, 
			IS_ALLOWED SMALLINT NOT NULL, 
			UNIQUE (PERMISSION_ID, USER_ID), 
			FOREIGN KEY (PERMISSION_ID) REFERENCES UM_PERMISSIONS(ID) ON DELETE CASCADE, 
			FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE, 
			PRIMARY KEY (ID));

CREATE TABLE UM_USER_ROLES (
			ID INTEGER GENERATED BY DEFAULT AS IDENTITY, 
			ROLE_ID INTEGER NOT NULL, 
			USER_ID INTEGER NOT NULL, 
			UNIQUE (USER_ID, ROLE_ID), 
			FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE, 
			FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE, 
			PRIMARY KEY (ID));
			
CREATE TABLE ASSOCIATION (
            ASSOCIATION_ID INTEGER GENERATED BY DEFAULT AS IDENTITY,
            RID1 INTEGER  NOT NULL,
            RID2 INTEGER  NOT NULL,
            ASSOCIATION_TYPE VARCHAR (20) NOT NULL,
            UNIQUE (RID1, RID2, ASSOCIATION_TYPE),
            FOREIGN KEY (RID1) REFERENCES ARTIFACTS (AID),FOREIGN KEY (RID2) REFERENCES ARTIFACTS (AID));